/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-

   this file is part of rcssserver3D
   Fri May 9 2003
   Copyright (C) 2002,2003 Koblenz University
   Copyright (C) 2003 RoboCup Soccer Server 3D Maintenance Group

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Main of the rcssmonitor3D application

   This is a very simple 3D monitor example which will hopefully be
   used for the first steps of the 3D SoccerServer
*/

/** This is a dummy agent created for testing with the SpadesTest. It
    reads length prefixed data (see spades manual 6.3) from fd 3 and
    writes to fd 4. A corresponding agent DB (agentdb.list, entry
    'default') exists in the spadestest directory.
*/
#include <sstream>
#include <zeitgeist/zeitgeist.h>
#include <oxygen/oxygen.h>
#include <soccer/soccertypes.h>
#include "worldmodel.h"
#include "commserver.h"
#include "behavior.h"
#include "soccer.h"
#include "kicknrun.h"

// commserver instance
static boost::shared_ptr< CommServer > G_commserver;

// worldmodel instance
static boost::shared_ptr< WorldModel > G_worldmodel;

// behavior instance
static boost::shared_ptr< Soccer > G_behavior;

// the default team name
std::string teamName = "TokyoTechSFC";

// the playes uniform number, if left to zero the server will allocate
// the next freee unum
int teamUnum = 0;

// set to 1 to write debug information to stdout
#define ENABLE_LOGGING 1

#ifdef ENABLE_LOGGING
/*-------------------------------------------------------------------*/
/*!

*/
void
Log( const char* out )
{
    std::printf( out );
    std::fflush( 0 );
}
#else
#define Log(x)
#endif

/*-------------------------------------------------------------------*/
/*!

*/
void
print_help()
{
    std::cout << "\nusage: agenttest [options]" << std::endl;
    std::cout << "\noptions:" << std::endl;
    std::cout << " --help      prints this message." << std::endl;
    std::cout << " --teamname  sets the team name. " << std::endl;
    std::cout << " --unum      sets the uniform number." << std::endl;
    std::cout << "\n";
}

/*-------------------------------------------------------------------*/
/*!

*/
void
read_options( int argc, char* argv[] )
{
    for ( int i = 0; i < argc; ++i )
    {
        if ( std::strcmp( argv[i], "--teamname" ) == 0 )
        {
            if( i + 1  < argc )
            {
                teamName = argv[i+1];
                ++i;
                Log("setting teamname to ");
                Log( teamName.c_str() );
                Log("\n");
            }
        }
        else if ( std::strcmp( argv[i], "--help" ) == 0 )
        {
            print_help();
            std::exit( 0 );
        }
        else if ( std::strcmp( argv[i], "--unum" ) == 0 )
        {
            teamUnum = std::atoi( argv[i+1] );
            ++i;
            std::stringstream ss;
            ss << "setting uniform number to " << teamUnum << "\n";
            Log( ss.str().c_str() );
        }
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
int
init( int argc, char* argv[], zeitgeist::Zeitgeist& zg )
{
    // register agent classes
    zg.GetCore()->RegisterClassObject( new CLASS(CommServer), "" );
    zg.GetCore()->RegisterClassObject( new CLASS(WorldModel), "" );
    zg.GetCore()->RegisterClassObject( new CLASS(KickNRun), "" );

    // run init script
    //zg.GetCore()->GetScriptServer()->RunInitScript("agent3d.rb", "agent3d/src");
    zg.GetCore()->GetScriptServer()->RunInitScript( "agenttest.rb", "./" );

    // read command line options
    read_options( argc,argv );

    // register and create the CommServer
    G_commserver = boost::shared_dynamic_cast< CommServer >
        ( zg.GetCore()->Get( "/sys/CommServer" ) );

    if ( G_commserver.get() == 0 )
    {
        Log("cannot find CommServer\n");
        return 1;
    }

    // register and create the WorldModel
    G_worldmodel
        = boost::shared_dynamic_cast< WorldModel >
        ( zg.GetCore()->Get( "/sys/WorldModel" ) );

    if ( G_worldmodel.get() == 0 )
    {
        Log( "cannot find WorldModel\n" );
        return 1;
    }

    // get the installed behavior
    G_behavior = boost::shared_dynamic_cast< Soccer >
        ( zg.GetCore()->Get( "sys/Behavior" ) );

    if ( G_behavior.get() == 0 )
    {
        Log( "cannot find Behavior\n" );
        return 1;
    }

    G_behavior->setTeamName( teamName );
    G_worldmodel->setTeamName( teamName );
    G_behavior->setTeamUnum( teamUnum );

    return 0;
}

/*-------------------------------------------------------------------*/
/*!

*/
void
main_loop()
{
    Log( "AgentTest started\n" );

    bool run = true;

    while ( run )
    {
        if ( ! G_commserver->getInput() )
        {
            continue;
        }
        // std::cout << "Msg von Server an Spieler: " <<comm->getMsg() <<"\n";
        G_worldmodel->parse( G_commserver->getMsg() );

        switch( G_commserver->getMsg()[0] ) {
        case 'D' :
            G_behavior->processInitMessage();
            break;
        case 'K' :
            G_behavior->processThinkTimeMessage();
            break;
        case 'S' :
            G_behavior->think();
            break;
        case 'X':
            run = false;
            Log("agent terminating\n");
            break;
        default:
            Log("received unknown data: ");
            Log( G_commserver->getMsg() );
            Log("\n");
            break;
        }
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
int
main( int argc, char* argv[] )
{
    //init zeitgeist and oxygen
    zeitgeist::Zeitgeist zg( ".rcssserver3d" );
    oxygen::Oxygen oyxgen( zg );

    int ret = init( argc, argv, zg );

    if ( ret > 0 )
    {
        return ret;
    }

    main_loop();
}
